# coding: utf-8

"""

    No descripton provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: 1.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from pprint import pformat
from six import iteritems
import re


class FeatureRequest(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """
    def __init__(self, locus=None, term=None, rank=None, sequence=None):
        """
        FeatureRequest - a model defined in Swagger

        :param dict swaggerTypes: The key is attribute name
                                  and the value is attribute type.
        :param dict attributeMap: The key is attribute name
                                  and the value is json key in definition.
        """
        self.swagger_types = {
            'locus': 'str',
            'term': 'str',
            'rank': 'int',
            'sequence': 'str'
        }

        self.attribute_map = {
            'locus': 'locus',
            'term': 'term',
            'rank': 'rank',
            'sequence': 'sequence'
        }

        self._locus = locus
        self._term = term
        self._rank = rank
        self._sequence = sequence

    @property
    def locus(self):
        """
        Gets the locus of this FeatureRequest.
        locus name or URI

        :return: The locus of this FeatureRequest.
        :rtype: str
        """
        return self._locus

    @locus.setter
    def locus(self, locus):
        """
        Sets the locus of this FeatureRequest.
        locus name or URI

        :param locus: The locus of this FeatureRequest.
        :type: str
        """

        self._locus = locus

    @property
    def term(self):
        """
        Gets the term of this FeatureRequest.
        Sequence Ontology (SO) term name, accession, or URI

        :return: The term of this FeatureRequest.
        :rtype: str
        """
        return self._term

    @term.setter
    def term(self, term):
        """
        Sets the term of this FeatureRequest.
        Sequence Ontology (SO) term name, accession, or URI

        :param term: The term of this FeatureRequest.
        :type: str
        """

        self._term = term

    @property
    def rank(self):
        """
        Gets the rank of this FeatureRequest.
        feature rank, must be at least 1

        :return: The rank of this FeatureRequest.
        :rtype: int
        """
        return self._rank

    @rank.setter
    def rank(self, rank):
        """
        Sets the rank of this FeatureRequest.
        feature rank, must be at least 1

        :param rank: The rank of this FeatureRequest.
        :type: int
        """

        self._rank = rank

    @property
    def sequence(self):
        """
        Gets the sequence of this FeatureRequest.
        feature sequence, in DNA alphabet

        :return: The sequence of this FeatureRequest.
        :rtype: str
        """
        return self._sequence

    @sequence.setter
    def sequence(self, sequence):
        """
        Sets the sequence of this FeatureRequest.
        feature sequence, in DNA alphabet

        :param sequence: The sequence of this FeatureRequest.
        :type: str
        """

        self._sequence = sequence

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other
