# coding: utf-8

"""

    No descripton provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: 1.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from __future__ import absolute_import

import os
import sys
import unittest

import swagger_client
from swagger_client.rest import ApiException
from swagger_client.apis.features_api import FeaturesApi


class TestFeaturesApi(unittest.TestCase):
    """ FeaturesApi unit test stubs """

    def setUp(self):
        self.api = swagger_client.apis.features_api.FeaturesApi()

    def tearDown(self):
        pass

    def test_create_feature(self):
        """
        Test case for create_feature

        Create an enumerated sequence feature
        """
        pass

    def test_get_feature_by_path(self):
        """
        Test case for get_feature_by_path

        Retrieve an enumerated sequence feature
        """
        pass

    def test_get_feature_by_query(self):
        """
        Test case for get_feature_by_query

        Retrieve an enumerated sequence feature
        """
        pass

    def test_list_features(self):
        """
        Test case for list_features

        List the enumerated sequence features at a locus
        """
        pass

    def test_list_features_0(self):
        """
        Test case for list_features_0

        List the enumerated sequence features matching a term at a locus
        """
        pass

    def test_list_features_1(self):
        """
        Test case for list_features_1

        List the enumerated sequence features matching a term and rank at a locus
        """
        pass


if __name__ == '__main__':
    unittest.main()
